@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301  USA.
 
    Author: Bastien Nocera <hadess@hadess.net>
  */
@@ -29,7 +29,6 @@
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include "totem-pl-parser.h"
 #include "totemplparser-marshal.h"
 #endif /* !TOTEM_PL_PARSER_MINI */
@@ -40,14 +39,16 @@
 
 #ifndef TOTEM_PL_PARSER_MINI
 
+#define SAFE_FREE(x) { if (x != NULL) xmlFree (x); }
+
 static xmlDocPtr
-totem_pl_parser_parse_xml_file (const char *url)
+totem_pl_parser_parse_xml_file (GFile *file)
 {
 	xmlDocPtr doc;
 	char *contents;
-	int size;
+	gsize size;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return NULL;
 
 	/* Try to remove HTML style comments */
@@ -75,11 +76,10 @@ totem_pl_parser_parse_xml_file (const char *url)
 gboolean
 totem_pl_parser_write_xspf (TotemPlParser *parser, GtkTreeModel *model,
 			   TotemPlParserIterFunc func, 
-			   const char *output, const char *title,
+			   GFile *output, const char *title,
 			   gpointer user_data, GError **error)
 {
-	GnomeVFSHandle *handle;
-	GnomeVFSResult res;
+	GFileOutputStream *stream;
 	int num_entries_total, num_entries, i;
 	char *buf;
 	gboolean success;
@@ -87,64 +87,48 @@ totem_pl_parser_write_xspf (TotemPlParser *parser, GtkTreeModel *model,
 	num_entries = totem_pl_parser_num_entries (parser, model, func, user_data);
 	num_entries_total = gtk_tree_model_iter_n_children (model, NULL);
 
-	res = gnome_vfs_open (&handle, output, GNOME_VFS_OPEN_WRITE);
-	if (res == GNOME_VFS_ERROR_NOT_FOUND) {
-		res = gnome_vfs_create (&handle, output,
-				GNOME_VFS_OPEN_WRITE, FALSE,
-				GNOME_VFS_PERM_USER_WRITE
-				| GNOME_VFS_PERM_USER_READ
-				| GNOME_VFS_PERM_GROUP_READ);
-	}
-
-	if (res != GNOME_VFS_OK) {
-		g_set_error(error,
-			    TOTEM_PL_PARSER_ERROR,
-			    TOTEM_PL_PARSER_ERROR_VFS_OPEN,
-			    _("Couldn't open file '%s': %s"),
-			    output, gnome_vfs_result_to_string (res));
+	stream = g_file_replace (output, NULL, FALSE, G_FILE_CREATE_NONE, NULL, error);
+	if (stream == NULL)
 		return FALSE;
-	}
 
 	buf = g_strdup_printf ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
 				"<playlist version=\"1\" xmlns=\"http://xspf.org/ns/0/\">\n"
 				" <trackList>\n");
-	success = totem_pl_parser_write_string (handle, buf, error);
+	success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 	g_free (buf);
 	if (success == FALSE)
-	{
-		gnome_vfs_close (handle);
 		return FALSE;
-	}
 
 	for (i = 1; i <= num_entries_total; i++) {
 		GtkTreeIter iter;
 		char *url, *url_escaped, *relative, *title;
+		GFile *file;
 		gboolean custom_title;
 
 		if (gtk_tree_model_iter_nth_child (model, &iter, NULL, i - 1) == FALSE)
 			continue;
 
 		func (model, &iter, &url, &title, &custom_title, user_data);
+		file = g_file_new_for_uri (url);
 
-		if (totem_pl_parser_scheme_is_ignored (parser, url) != FALSE)
-		{
+		if (totem_pl_parser_scheme_is_ignored (parser, url) != FALSE) {
+			g_object_unref (file);
 			g_free (url);
 			g_free (title);
 			continue;
 		}
+		g_object_unref (file);
 
-		relative = totem_pl_parser_relative (url, output);
+		relative = totem_pl_parser_relative (output, url);
 		url_escaped = g_markup_escape_text (relative ? relative : url, -1);
 		buf = g_strdup_printf ("  <track>\n"
 					"   <location>%s</location>\n", url_escaped);
-		success = totem_pl_parser_write_string (handle, buf, error);
+		success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 		g_free (url);
 		g_free (url_escaped);
 		g_free (relative);
 		g_free (buf);
-		if (success == FALSE)
-		{
-			gnome_vfs_close (handle);
+		if (success == FALSE) {
 			g_free (title);
 			return FALSE;
 		}
@@ -155,36 +139,42 @@ totem_pl_parser_write_xspf (TotemPlParser *parser, GtkTreeModel *model,
 		else
 			buf = g_strdup_printf ("  </track>\n");
 		
-		success = totem_pl_parser_write_string (handle, buf, error);
+		success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 		g_free (buf);
 		g_free (title);
 		if (success == FALSE)
-		{
-			gnome_vfs_close (handle);
 			return FALSE;
-		}
 	}
 
 	buf = g_strdup_printf (" </trackList>\n"
 				"</playlist>");
-	success = totem_pl_parser_write_string (handle, buf, error);
+	success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 	g_free (buf);
-	gnome_vfs_close (handle);
+
+	g_output_stream_close (G_OUTPUT_STREAM (stream), NULL, NULL);
 
 	return success;
 }
 
 static gboolean
-parse_xspf_track (TotemPlParser *parser, char *base, xmlDocPtr doc,
+parse_xspf_track (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
 		xmlNodePtr parent)
 {
 	xmlNodePtr node;
-	xmlChar *title, *url;
-	gchar *fullpath;
+	xmlChar *title, *url, *image_url, *artist, *album, *duration, *moreinfo;
+	xmlChar *download_url, *id;
+	GFile *resolved;
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_ERROR;
 	
 	title = NULL;
 	url = NULL;
+	image_url = NULL;
+	artist = NULL;
+	album = NULL;
+	duration = NULL;
+	moreinfo = NULL;
+	download_url = NULL;
+	id = NULL;
 
 	for (node = parent->children; node != NULL; node = node->next)
 	{
@@ -193,32 +183,73 @@ parse_xspf_track (TotemPlParser *parser, char *base, xmlDocPtr doc,
 
 		if (g_ascii_strcasecmp ((char *)node->name, "location") == 0)
 			url = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
-
-		if (g_ascii_strcasecmp ((char *)node->name, "title") == 0)
+		else if (g_ascii_strcasecmp ((char *)node->name, "title") == 0)
 			title = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
+		else if (g_ascii_strcasecmp ((char *)node->name, "image") == 0)
+			image_url = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
+		/* Last.fm uses creator for the artist */
+		else if (g_ascii_strcasecmp ((char *)node->name, "creator") == 0)
+			artist = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
+		else if (g_ascii_strcasecmp ((char *)node->name, "duration") == 0)
+			duration = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
+		else if (g_ascii_strcasecmp ((char *)node->name, "link") == 0) {
+			xmlChar *rel;
+
+			rel = xmlGetProp (node, (const xmlChar *) "rel");
+			if (rel != NULL) {
+				if (g_ascii_strcasecmp ((char *) rel, "http://www.last.fm/trackpage") == 0)
+					moreinfo = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
+				else if (g_ascii_strcasecmp ((char *) rel, "http://www.last.fm/freeTrackURL") == 0)
+					download_url = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
+				xmlFree (rel);
+			} else {
+				/* If we don't have a rel="", then it's not a last.fm playlist */
+				moreinfo = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
+			}
+		} else if (g_ascii_strcasecmp ((char *)node->name, "album") == 0)
+			album = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
+		else if (g_ascii_strcasecmp ((char *)node->name, "trackauth") == 0)
+			id = xmlNodeListGetString (doc, node->xmlChildrenNode, 1);
 	}
 
 	if (url == NULL) {
-		if (title)
-			xmlFree (title);
-		return TOTEM_PL_PARSER_RESULT_ERROR;
+		retval = TOTEM_PL_PARSER_RESULT_ERROR;
+		goto bail;
 	}
 
-	fullpath = totem_pl_parser_resolve_url (base, (char *)url);
-	totem_pl_parser_add_one_url (parser, fullpath, (char *)title);
+	if (base_file != NULL && strstr (url, "://") == NULL)
+		resolved = g_file_resolve_relative_path (base_file, (const char *) url);
+	else
+		resolved = g_file_new_for_uri ((const char *) url);
+	totem_pl_parser_add_url (parser,
+				 TOTEM_PL_PARSER_FIELD_FILE, resolved,
+				 TOTEM_PL_PARSER_FIELD_TITLE, title,
+				 TOTEM_PL_PARSER_FIELD_DURATION_MS, duration,
+				 TOTEM_PL_PARSER_FIELD_IMAGE_URL, image_url,
+				 TOTEM_PL_PARSER_FIELD_AUTHOR, artist,
+				 TOTEM_PL_PARSER_FIELD_ALBUM, album,
+				 TOTEM_PL_PARSER_FIELD_MOREINFO, moreinfo,
+				 TOTEM_PL_PARSER_FIELD_DOWNLOAD_URL, download_url,
+				 TOTEM_PL_PARSER_FIELD_ID, id,
+				 NULL);
+	g_object_unref (resolved);
+
 	retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 
-	if (title)
-		xmlFree (title);
-	if (url)
-		xmlFree (url);
-	g_free (fullpath);
+bail:
+	SAFE_FREE (title);
+	SAFE_FREE (url);
+	SAFE_FREE (image_url);
+	SAFE_FREE (artist);
+	SAFE_FREE (album);
+	SAFE_FREE (duration);
+	SAFE_FREE (moreinfo);
 
 	return retval;
 }
 
 static gboolean
-parse_xspf_trackList (TotemPlParser *parser, char *base, xmlDocPtr doc,
+parse_xspf_trackList (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
 		xmlNodePtr parent)
 {
 	xmlNodePtr node;
@@ -230,7 +261,7 @@ parse_xspf_trackList (TotemPlParser *parser, char *base, xmlDocPtr doc,
 			continue;
 
 		if (g_ascii_strcasecmp ((char *)node->name, "track") == 0)
-			if (parse_xspf_track (parser, base, doc, node) != FALSE)
+			if (parse_xspf_track (parser, base_file, doc, node) != FALSE)
 				retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
 
@@ -238,7 +269,7 @@ parse_xspf_trackList (TotemPlParser *parser, char *base, xmlDocPtr doc,
 }
 
 static gboolean
-parse_xspf_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
+parse_xspf_entries (TotemPlParser *parser, GFile *base_file, xmlDocPtr doc,
 		xmlNodePtr parent)
 {
 	xmlNodePtr node;
@@ -249,7 +280,7 @@ parse_xspf_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 			continue;
 
 		if (g_ascii_strcasecmp ((char *)node->name, "trackList") == 0)
-			if (parse_xspf_trackList (parser, base, doc, node) != FALSE)
+			if (parse_xspf_trackList (parser, base_file, doc, node) != FALSE)
 				retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
 
@@ -257,15 +288,16 @@ parse_xspf_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 }
 
 TotemPlParserResult
-totem_pl_parser_add_xspf (TotemPlParser *parser, const char *url,
-			  const char *_base, gpointer data)
+totem_pl_parser_add_xspf (TotemPlParser *parser,
+			  GFile *file,
+			  GFile *base_file,
+			  gpointer data)
 {
 	xmlDocPtr doc;
 	xmlNodePtr node;
-	char *base;
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
-	doc = totem_pl_parser_parse_xml_file (url);
+	doc = totem_pl_parser_parse_xml_file (file);
 
 	/* If the document has no root, or no name */
 	if(!doc || !doc->children
@@ -277,13 +309,11 @@ totem_pl_parser_add_xspf (TotemPlParser *parser, const char *url,
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
 
-	base = totem_pl_parser_base_url (url);
-
-	for (node = doc->children; node != NULL; node = node->next)
-		if (parse_xspf_entries (parser, base, doc, node) != FALSE)
+	for (node = doc->children; node != NULL; node = node->next) {
+		if (parse_xspf_entries (parser, base_file, doc, node) != FALSE)
 			retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
+	}
 
-	g_free (base);
 	xmlFreeDoc(doc);
 	return retval;
 }
